@@ -1,6 +1,6 @@
 /* 
    cadaver, command-line DAV client
-   Copyright (C) 1999-2001, Joe Orton <joe@manyfish.co.uk>, 
+   Copyright (C) 1999-2004, Joe Orton <joe@manyfish.co.uk>, 
    except where otherwise indicated.
                                                                      
    This program is free software; you can redistribute it and/or modify
@@ -1113,16 +1113,16 @@ static int supply_creds(const char *prompt, const char *realm, const char *hostn
 static int supply_creds_server(void *userdata, const char *realm, int attempt,
 			       char *username, char *password)
 {
-    if (attempt > 1)
-	return -1;
-
-    /* Pass back netrc values if we have them. */
-    if (server_username != NULL && server_password != NULL) {
+    /* Try netrc creds if we have them on first auth attempt. */
+    if (server_username && server_password && attempt-- == 0) {
 	ne_strnzcpy(username, server_username, NE_ABUFSIZ);
 	ne_strnzcpy(password, server_password, NE_ABUFSIZ);
 	return 0;
     }
 
+    if (attempt > 1)
+	return -1;
+
     return supply_creds(
 	_("Authentication required for %s on server `%s':\n"), realm,
 	server.host, username, password);
